<?php

namespace Insallah;

class TournamentModel
{

    /**
     *  ,  ,  ,   
     *   30 
     *
     * @param int $uid
     *
     * @return int
     */
    public static function getUserLevel($uid)
    {
        $db = new Db();
        $level = $db::getValue('select level from users where id = ? and level between 8 and 12 and battle = 0', [$uid]);
        return $level ?: 0;
    }

    /**
     * @param int $uid
     * @param int $level
     *
     * @return bool
     */
    public static function isEkrOverpriced($uid, $level = null)
    {
        $db = new Db();
        if (is_null($level)) {
            $level = $db::getValue('select level from users where id = ?', [$uid]);
        }
        $wearedItemsEkrPrice = $db::getValue('select sum(2price) from items_users where inOdet > 0 and uid = ?', [$uid]);
        return $wearedItemsEkrPrice > Tournament::ekrOverpriceFormula($level);
    }

    /**
     * @param int $uid
     *
     * @return bool
     */
    public static function isEnoughExperience($uid)
    {
        $db = new Db();
        return $db::getValue('select exp from stats where id = ?', [$uid]) >= Tournament::MIN_EXP;
        }

    /**
     * @param int $uid
     *
     * @return bool
     */
    public static function IsRestrictedToJoin($uid)
    {
        $db = new Db();
        $delayEffect = $db::getValue('select count(*) from eff_users where uid = ? and id_eff = 486 and `delete` = 0', [$uid]);
        return (bool)$delayEffect;
    }

    /**
     * @param $tid
     *
     * @return bool
     */
    public static function isStarted($tid)
    {
        $db = new Db();
        $status = $db::getValue('select count(*) from tournaments where start_time = -1 and tid = ?', [$tid]);
        return (bool)$status;
    }

    /**
     *        .
     *
     * @param int $tid
     *
     * @return int
     */
    public static function getWaitingMembersQuantity($tid)
    {
        $db = new Db();
        return $db::getValue('select count(*) from tournaments_users where tid = ?', [$tid]);
    }

    /**
     *   .
     *
     * @param int $tid
     *
     * @return void
     */
    public static function createTournament($tid)
    {
        $db = new Db();
        $db::sql('insert into tournaments (tid) values (?)', [$tid]);
    }

    /**
     *         .
     *
     * @param int $uid
     * @param int $tid
     *
     * @return void
     */
    public static function joinTournament($uid, $tid)
    {
        /**   25008 - 25012. */
        $room_id = 25000 + $tid;
        $db = new Db();
        $db::sql('insert into tournaments_users (tid, uid) values (?, ?)', [$tid, $uid]);
        self::teleport($uid, $room_id);
    }

    /**
     *  .
     *
     * @param int $tid
     *
     * @return void
     */
    public static function startTournament($tid)
    {
        $db = new Db();
        $db::sql('update tournaments set start_time = -1 where tid = ?', [$tid]);
    }

    /**
     *     .
     *
     * @param int $tid
     *
     * @return void
     */
    public static function destroyTournament($tid)
    {
        $db = new Db();
        //     foreign_keys    tournaments_users !!!
        $db::sql('delete from tournaments where tid = ?', [$tid]);
    }

    /**
     *        .    + 1     .
     *
     * @param array $fighters_list
     *
     * @return array
     */
    public static function getFightersTeams(array $fighters_list)
    {
        $db = new Db();
        $query = sprintf("select id from users where battle = 0 and id in (%s)", implode(', ', $fighters_list));
        return array_chunk($db::getColumn($query), 2);
    }

    /**
     *        .
     *
     * @param int $tid
     *
     * @return array
     */
    public static function getFreeFighters($tid)
    {
        $db = new Db();
        return $db::getColumn('select uid from tournaments_users where tid = ? and death_time = 0 order by uid', [$tid]);
    }

    /**
     *  .  ,     .
     *
     * @param int $tid
     *
     * @return array
     */
    public static function getWinners($tid)
    {
        $db = new Db();
        $winners = $db::getColumn('select uid from tournaments_users where tid = ? order by death_time desc limit 3', [$tid]);
        return [
            1 => $winners[0],
            2 => $winners[1],
            3 => $winners[2]
        ];
    }

    /**
     *              .
     * @return mixed
     */
    public static function getLooser()
    {
        $query = '
            select uid, battle 
            from 
                battle_users, 
                (select id, team_win 
                 from battle 
                 where 
                     team_win > 0 and 
                     typeBattle = 25000 
                 order by time_over desc 
                 limit 1) as last_battle 
            where 
                battle_users.battle = last_battle.id and 
                battle_users.team != last_battle.team_win and
                battle_users.uid in (select uid from tournaments_users where death_time = 0)';

        $query2 = 'select bu.uid from battle b
                    inner join battle_users bu on b.team_win != bu.team and b.id = bu.battle
                    inner join tournaments_users tu on bu.uid = tu.uid
                    where typeBattle = 25000 and death_time = 0 order by b.time_start desc limit 1';
        $db = new Db;
        $row = $db::getRow($query);
        return $row['uid'];
    }

    /**
     *         .
     *
     * @param int $uid
     *
     * @return void
     */
    public static function removeFighter($uid, $winner = false)
    {
        if (!$uid) return;
        //$winner_timer_add = $winner? 500 : 0; #    .
        $db = new Db();
        $db::sql('update tournaments_users set death_time = unix_timestamp() + 500 where death_time = 0 and uid = ?', [$uid]);
        self::teleport($uid, 9);
        //fixme:      .    . :(
        Db::sql("update users_achiv set trn = trn + 1 where id = ?", [$uid]);
        //(new Achievements(\user::start()))->updateCounter('trn');
    }

    /**
     *  id   id .
     *
     * @param int $uid
     *
     * @return mixed
     */
    public static function getTournamentIdByUserId($uid)
    {
        $db = new Db();
        return $db::getValue('select tid from tournaments_users where uid = ?', [$uid]);
    }

    /**
     *   .
     * 25000 -  id   .
     * noinc -   
     * invis -  
     *
     * @param int $uid1
     * @param int $uid2
     *
     * @return void
     */
    public static function startBattle($uid1, $uid2)
    {
        $db = new Db();
        $check = Db::getValue('select count(*) from users where id in (?, ?) and battle = 0', [$uid1, $uid2]);
        if ((int)$check !== 2) {
            return;
        }

        $db::exec('insert into battle (city, time_start, timeout, type, invis, noinc, travmChance, typeBattle) 
        values (\'capitalcity\', unix_timestamp(), 60, 0, 1, 1, 0, 25000)');
        $bid = $db::lastInsertId(); // !
        $db::sql('update stats set team = 1, hpNow = hpAll, mpNow = mpAll where id = ?', [$uid1]);
        $db::sql('update stats set team = 2, hpNow = hpAll, mpNow = mpAll where id = ?', [$uid2]);
        $db::sql('update users set battle = ? where id in (?, ?)', [$bid, $uid1, $uid2]);
    }

    /**
     *      id.
     *
     * @param int $uid
     *
     * @return mixed
     */
    public static function uidToLogin($uid)
    {
        $db = new Db();
        return $db::getValue('select login from users where id = ?', [$uid]);
    }

    /**
     *   .
     *
     * @param int $uid
     * @param int $room_id
     *
     * @return void
     */
    private static function teleport($uid, $room_id)
    {
        $db = new Db();
        $db::sql('update users set room = ? where id = ?', [$room_id, $uid]);
    }

    /**
     *   $message             .
     *
     * @param string $message
     *
     * @return void
     */
    public static function sysMessage($message)
    {
        if (!empty($message)) {
        $db = new Db();
            $message = "<span style='font-weight: bold; color: forestgreen;'>$message</span>";
            $db::sql('insert into chat (time, type, text, new, da) values (unix_timestamp(), 6, ?, 1, 1)', [$message]);
        }
    }

    /**
     *      $quantity      $uid.
     *
     * @param int $uid
     * @param int $quantity
     *
     * @return void
     */
    public static function givePrizeItems($uid, $quantity)
    {
        $query = 'insert into items_users (item_id, uid, data, iznosMAX, lastUPD, time_create) 
                    values (4754, :uid, :data, 1, unix_timestamp(), unix_timestamp())';
        $args = [
            'uid' => $uid,
            'data' => 'nosale=1|musor=1|sudba=' . self::uidToLogin($uid) . '|lvl=8|tr_s1=0|tr_s2=0|tr_s3=0|tr_s4=0'
        ];
        $db = new Db();
        $stmt = $db::prepare($query);
        for ($i = 0; $i < $quantity; $i++) {
            $stmt->execute($args);
        }
    }

    /** -    .
     * @param $uid
     * @param $unix_time
     *
     * @return void
     */
    public static function giveDelay($uid, $unix_time)
    {
        $db = new Db();
        $query = 'insert into eff_users (id_eff, uid, name, timeUse) VALUES (?,?,?,?)';
        $args = [486, $uid, '  !', $unix_time];
        $db::sql($query, $args);
    }
}